<?php
  /*
* $Id: image_helper.php
* $Loc: /includes/system/override/kiss_image_thumbnailer/
*
* Name: KissitImage
* Version: 4.7.1
* Release Date: 11/29/2024
* Author: Rainer Schmied
* 	 phoenixcartaddonsaddons.com / raiwa@phoenixcartaddons.com
*
* License: Released under Commercial License
*
* Comments:  Author: [Rainer Schmied @raiwa]
*  Author URI: [www.phoenixcartaddons.com]
*  License: Commercial License Rainer Schmied
*  License URI: https://www.phoenixcartaddons.com/license.php
* 
*  Copyright (c) 2024 Rainer Schmied
*  
* Based on:
* S. Mohammed Alsharaf, http://www.zfsnippets.com/snippets/view/id/44.
* Copyright 2008-2009 FWR Media ( Robert Fisher )
* 
* Author Robert Fisher, FWR Media, http://www.fwrmedia.co.uk
*  Redistribution and use in source and binary forms, with or without
*  modification, are not permitted without specific prior written permission
*  from Rainer Schmied
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
*  SOFTWARE.
*  
* 
*/
  require_once DIR_FS_CATALOG . 'includes/system/override/kiss_image_thumbnailer/image_kissit.php';

  /**
  * Helper class to create valid thumbnails on the fly within the tep_image() wrapper
  *
  *
  * @license    http://www.gnu.org/licenses/gpl-2.0.html GNU Public License)
  * @author     Robert fisher - FWR Media ( www.fwrmedia.co.uk )
  * @version     1.0
  */
  class image_helper extends ArrayObject {
    /**
    * put your comment there...
    *
    * @var mixed
    */
    protected $_valid_mime = ['image/png', 'image/jpeg', 'image/jpg', 'image/gif', 'image/webp'];
    /**
    * put your comment there...
    *
    */
    public function __construct( $input ) {
      parent::__construct( $input, parent::ARRAY_AS_PROPS );
    } // end constructor
    /**
    * put your comment there...
    *
    */

    public static function generate_thumb(&$image) {

      $attributes = ['width' => $image->get('width'), 'height' => $image->get('height'), 'watermark' => $image->get('watermark')];

      $kiss_image = new image_helper(['src' => $image->get('src'),
                                      'attributes' => $attributes
                                      ]);


      if ( false !== $image_assembled = $kiss_image->assemble() ) {
        // admin  support change system path back to web path
        if (Text::is_prefixed_by($kiss_image['src'], DIR_FS_CATALOG)) {
          $kiss_image['src'] = HTTP_SERVER . DIR_WS_CATALOG . Text::ltrim_once($kiss_image['src'], DIR_FS_CATALOG);
        }

        $image->set('src', $kiss_image['src']);
        $image->set('width', (int)$kiss_image->_calculated_width);
        $image->set('height', (int)$kiss_image->_calculated_height);
      }
    } // end method
    /**
    * put your comment there...
    *  // end method
    * @image from image class
    */

    protected function assemble() {
      $image_check = $this->_checkImage();
      // Image check is bad so we pass control back to the old OsC image wrapper
      if ( 'abort' == $image_check ) {
        return false;
      }

      // If we have to only we generate a thumb .. this is very resource intensive
      if ( 'no_thumb_required' !== $image_check ) {
        $this->_generateThumbnail();
      }
    } // end method

    /**
    * put your comment there...
    *  // end method
    * @param mixed $attribs
    */
    protected function _checkImage() {
      $this->thumbs_dir_path = str_replace('images/', 'images/' . KISSIT_THUMBS_MAIN_DIR . $this->attributes['width'] .'_'. $this->attributes['height'] . '/', dirname($this->src) . '/');
      // admin  support change system path back to web path
      $thumbs_dir_path = $this->thumbs_dir_path;
      if (Text::is_prefixed_by($thumbs_dir_path, HTTP_SERVER . DIR_WS_CATALOG)) {
        $this->thumbs_dir_path = DIR_FS_CATALOG . Text::ltrim_once($thumbs_dir_path, HTTP_SERVER . DIR_WS_CATALOG);
      }

      $this->thumb_background_rgb = array_combine(['red', 'green', 'blue'], explode (',', defined('KISSIT_BACKGROUND_COLOR')? KISSIT_BACKGROUND_COLOR: ('255,255,255')));
      $image_path_parts = pathinfo ( $this->src );
      $this->_image_name = $image_path_parts['basename'];
      $this->_original_image_info = $this->src;
      $this->_thumb_filename = str_replace(' ', '-', $this->_image_name);
      $this->_thumb_src = $this->thumbs_dir_path . $this->_thumb_filename;

      if ( is_readable ( $this->_thumb_src ) ) {
        $this->_calculated_width = $this->attributes['width'];
        $this->_calculated_height = $this->attributes['height'];
        $this->src = $this->_thumb_src;
        return 'no_thumb_required';
      }
      if ( KISSIT_MAIN_PRODUCT_WATERMARK_SIZE == 0 ) {
      	if ( !$this->_original_image_info = getimagesize ( $this->src ) ) {
      		return 'abort';
      	}
      	if (!in_array ( $this->_original_image_info['mime'], $this->_valid_mime ) ) {
      		return 'abort';
      	}
      }
    } // end method
    /**
    * put your comment there...
    *
    */
    protected function _generateThumbnail() {
      if ( $this->attributes['width'] == $this->_original_image_info[0] && $this->attributes['height'] == $this->_original_image_info[1] ) {
        $this->_calculated_width = $this->attributes['width'];
        return $this->_calculated_height = $this->attributes['height'];
      }

      if ( $this->attributes['width'] == 0 || $this->attributes['height'] == 0 ) {
        $this->_calculated_width =  $this->_original_image_info[0];
        return $this->_calculated_height = $this->_original_image_info[1];
      }

      // make the required subdirectories
      if (!is_dir($this->thumbs_dir_path)) {
        mkdir($this->thumbs_dir_path, 0775, true); // true ->Allows the creation of nested directories specified in the pathname
      }

      // resize image
      $kiss_image = new Image_Kissit();
      $kiss_image->open( $this->src, $this->thumb_background_rgb )
            ->resize( (int)$this->attributes['width'], (int)$this->attributes['height'], $this->attributes['watermark'] )
            ->save( $this->_thumb_src, (int)KISSIT_JPEG_QUALITY );
      $this->_thumbnail = $kiss_image;
      $this->_calculated_width = $this->_thumbnail->getWidth();
      $this->_calculated_height = $this->_thumbnail->getHeight();
      $this->src = $this->_thumb_src;
    } // end method

  } // end class